@@ -53,13 +53,14 @@ grib_option grib_options[]={
         {"T:",0,0,1,0,"B"},
         {"7",0,0,0,1,0},
         {"V",0,0,0,1,0},
-        {"q",0,0,1,0,0}
+        {"q",0,0,1,0,0},
+        {"X:",0,0,0,1,0}
         /* {"x",0,0,0,1,0} */
 };
 
-char* grib_tool_description="Dump the content of a BUFR file in different formats.";
-char* grib_tool_name="bufr_dump";
-char* grib_tool_usage="[options] bufr_file bufr_file ...";
+const char* grib_tool_description="Dump the content of a BUFR file in different formats.";
+const char* grib_tool_name="bufr_dump";
+const char* grib_tool_usage="[options] bufr_file bufr_file ...";
 static int json=0;
 static char* json_option=0;
 static int first_handle=1;
@@ -82,6 +83,18 @@ int grib_tool_before_getopt(grib_runtime_options* options)
     return 0;
 }
 
+static void check_code_gen_dump_mode(const char* language)
+{
+    grib_context *c = grib_context_get_default();
+    const int ok = (strcmp(language, "C")==0       ||
+                    strcmp(language, "fortran")==0 ||
+                    strcmp(language, "python")==0  ||
+                    strcmp(language, "filter")==0);
+    if (!ok) {
+        grib_context_log(c, GRIB_LOG_ERROR, "Invalid language specified. Select one of: filter, fortran, python or C");
+    }
+}
+
 int grib_tool_init(grib_runtime_options* options)
 {
     int opt=grib_options_on("C")+grib_options_on("O");
@@ -119,10 +132,12 @@ int grib_tool_init(grib_runtime_options* options)
 
     if (grib_options_on("D:")) {
         options->dump_mode = grib_options_get_option("D:");
+        check_code_gen_dump_mode(options->dump_mode);
         json=0;
     }
     if (grib_options_on("E:")) {
         options->dump_mode = grib_options_get_option("E:");
+        check_code_gen_dump_mode(options->dump_mode);
         json=0;
     }
 
@@ -280,7 +295,7 @@ int grib_tool_new_handle_action(grib_runtime_options* options, grib_handle* h)
                 } else {
                     fprintf(stdout,"\"ERROR: unable to unpack data section\"");
                     options->error=err;
-                    return err;
+                    /*return err; See ECC-723*/
                 }
             }
             a=grib_find_accessor(h,"numericValues");
@@ -298,7 +313,7 @@ int grib_tool_new_handle_action(grib_runtime_options* options, grib_handle* h)
                 } else {
                     fprintf(stdout,"\"ERROR: unable to unpack data section\"");
                     options->error=err;
-                    return err;
+                    /*return err; See ECC-723*/
                 }
             }
             grib_dump_content(h,stdout,options->dump_mode,options->dump_flags,0);
@@ -313,7 +328,7 @@ int grib_tool_new_handle_action(grib_runtime_options* options, grib_handle* h)
                 } else {
                     fprintf(stdout,"\"ERROR: unable to unpack data section\"");
                     options->error=err;
-                    return err;
+                    /*return err; See ECC-723*/
                 }
             }
             options->dump_flags=GRIB_DUMP_FLAG_ALL_ATTRIBUTES;
@@ -337,14 +352,18 @@ int grib_tool_new_handle_action(grib_runtime_options* options, grib_handle* h)
                 fprintf(stderr, "ERROR: unable to unpack data section: %s\n",grib_get_error_message(err));
                 exit(err);
             } else {
-                fprintf(stdout,"\"ERROR: unable to unpack data section\"");
+                fprintf(stdout,"ERROR: unable to unpack data section\n");
                 options->error=err;
-                return err;
+                /*return err; See ECC-723*/
             }
         }
         grib_dump_content(h,stdout,options->dump_mode,options->dump_flags,0);
     } else {
         const char* dumper_name = get_dumper_name(options);
+        if (strcmp(dumper_name, "bufr_simple")==0) {
+            /* This speeds up the unpack by skipping attribute keys not used in the dump */
+            grib_set_long(h, "skipExtraKeyAttributes", 1);
+        }
         err=grib_set_long(h,"unpack",1);
         if (err) {
             if (options->fail) {
@@ -353,7 +372,7 @@ int grib_tool_new_handle_action(grib_runtime_options* options, grib_handle* h)
             } else {
                 fprintf(stdout,"\"ERROR: unable to unpack data section\"");
                 options->error=err;
-                return err;
+                /*return err; See ECC-723*/
             }
         }
         print_header(options);
@@ -429,7 +448,7 @@ int grib_tool_finalise_action(grib_runtime_options* options)
             fprintf(stdout,"    f.close()\n\n");
             fprintf(stdout,"def main():\n");
             fprintf(stdout,"    if len(sys.argv) < 2:\n");
-            fprintf(stdout,"        print >>sys.stderr, 'Usage: ', sys.argv[0], ' BUFR_file'\n");
+            fprintf(stdout,"        print('Usage: ', sys.argv[0], ' BUFR_file', file=sys.stderr)\n");
             fprintf(stdout,"        sys.exit(1)\n\n");
             fprintf(stdout,"    try:\n");
             fprintf(stdout,"        bufr_decode(sys.argv[1])\n");
