@@ -219,9 +219,13 @@ static int grib_tool_with_orderby(grib_runtime_options* options)
     }
 
     options->handle_count=0;
+    grib_context_set_handle_file_count(c, 0); /* ECC-873 */
+    grib_context_set_handle_total_count(c, 0); /* ECC-873 */
     while(!options->skip_all && ((h = grib_fieldset_next_handle(set,&err))
             != NULL || err != GRIB_SUCCESS )) {
         options->handle_count++;
+        grib_context_set_handle_file_count(c, options->handle_count);/* ECC-873 */
+        grib_context_set_handle_total_count(c, options->handle_count);/* ECC-873 */
         options->error=err;
 
         if (!h) {
@@ -532,38 +536,32 @@ static int scan(grib_context* c,grib_runtime_options* options,const char* dir)
             process(c,options,buf);
         }
     }
+    closedir(d);
     return 0;
 }
 #else
-static int isWinDir(const struct _finddata_t *fileinfo)
-{
-    if((fileinfo->attrib & 16) == 16)
-        return 1;
-    return 0;
-}
-static void doProcessing(grib_context* c,grib_runtime_options* options,const char* dir, const struct _finddata_t *fileinfo)
-{
-    if(isWinDir(fileinfo))
-    {
-        if(strcmp(fileinfo->name, ".") != 0 && strcmp(fileinfo->name,"..") != 0) {
-            char buf[1024];
-            sprintf(buf,"%s/%s",dir,fileinfo->name);
-            process(c,options,buf);
-        }
-    }
-}
 static int scan(grib_context* c,grib_runtime_options* options,const char* dir) {
     struct _finddata_t fileinfo;
     intptr_t handle;
-    if((handle = _findfirst(dir, &fileinfo)) != -1)
-        doProcessing(c, options, dir, &fileinfo);
+    char buffer[1024];
+    sprintf(buffer,  "%s/*", dir);
+    if((handle = _findfirst(buffer, &fileinfo)) != -1)
+{
+        do {
+            if(strcmp(fileinfo.name, ".") != 0 && strcmp(fileinfo.name,"..") != 0) {
+            char buf[1024];
+                sprintf(buf, "%s/%s", dir, fileinfo.name);
+            process(c,options,buf);
+        }
+        } while(!_findnext(handle, &fileinfo));
+
+        _findclose(handle);
+    }
     else
     {
         grib_context_log(c,(GRIB_LOG_ERROR) | (GRIB_LOG_PERROR) , "opendir %s",dir);
         return GRIB_IO_PROBLEM;
     }
-    while(!_findnext(handle, &fileinfo))
-        doProcessing(c, options, dir, &fileinfo);
     return 0;
 }
 #endif
@@ -824,6 +822,54 @@ static int fix_for_lsdate_needed(grib_handle* h)
     return 0;
 }
 
+static int get_initial_element_of_array(grib_handle* h, const char* keyName, size_t num_vals, char* value)
+{
+    int err = 0, type = 0;
+    size_t len = num_vals;
+    char* sval = NULL;
+    unsigned char *uval = NULL;
+    long *lval = NULL;
+    double* dval = NULL;
+    grib_context* c = h->context;
+
+    Assert(num_vals > 1); /* This is for array keys */
+    if ((err = grib_get_native_type(h,keyName,&type))!=GRIB_SUCCESS) return err;
+    switch(type) {
+        case GRIB_TYPE_STRING:
+            grib_get_string_length(h,keyName,&len);
+            sval = (char*)grib_context_malloc(c,len*sizeof(char));
+            if (!sval) return GRIB_OUT_OF_MEMORY;
+            if((err = grib_get_string(h,keyName,sval,&len)) != GRIB_SUCCESS) return err;
+            sprintf(value, "%s", sval);
+            free(sval);
+            break;
+        case GRIB_TYPE_LONG:
+            lval = (long*)grib_context_malloc(c,num_vals*sizeof(long));
+            if (!lval) return GRIB_OUT_OF_MEMORY;
+            if((err = grib_get_long_array(h,keyName,lval,&len)) != GRIB_SUCCESS) return err;
+            sprintf(value, "%ld...", lval[0]);
+            free(lval);
+            break;
+        case GRIB_TYPE_DOUBLE:
+            dval = (double*)grib_context_malloc(c,num_vals*sizeof(double));
+            if (!dval) return GRIB_OUT_OF_MEMORY;
+            if((err = grib_get_double_array(h,keyName,dval,&len)) != GRIB_SUCCESS) return err;
+            sprintf(value, "%g...", dval[0]);
+            free(dval);
+            break;
+        case GRIB_TYPE_BYTES:
+            uval = (unsigned char*)grib_context_malloc(c,num_vals*sizeof(unsigned char));
+            if (!uval) return GRIB_OUT_OF_MEMORY;
+            if((err = grib_get_bytes(h,keyName,uval,&len)) != GRIB_SUCCESS) return err;
+            sprintf(value, "%d...", (short)uval[0]);
+            free(uval);
+            break;
+        default:
+            sprintf(value, "%s...", "");
+    }
+    return GRIB_SUCCESS;
+}
+
 void grib_print_key_values(grib_runtime_options* options, grib_handle* h)
 {
     int i=0;
@@ -835,6 +881,7 @@ void grib_print_key_values(grib_runtime_options* options, grib_handle* h)
     const char* notfound="not_found";
     int written_to_dump = 0; /* boolean */
     grib_accessor* acc = NULL;
+    size_t num_vals = 0;
 
     if (!options->verbose) return;
 
@@ -890,7 +937,12 @@ void grib_print_key_values(grib_runtime_options* options, grib_handle* h)
                     if (fix_lsdate && strcmp(pName, "date")==0) { /* ECC-707 */
                         pName = "ls.date";
                     }
-                    ret=grib_get_string( h,pName,value,&len);
+                    ret = grib_get_size(h, pName, &num_vals);
+                    if(ret==GRIB_SUCCESS && num_vals>1) { /* See ECC-278 */
+                        ret = get_initial_element_of_array(h, pName, num_vals, value);
+                    } else {
+                        ret=grib_get_string( h,pName,value,&len);
+                    }
                     break;
                 case GRIB_TYPE_DOUBLE:
                     ret=grib_get_double( h,options->print_keys[i].name,&dvalue);
